package org.puremvc.as3.multicore.utilities.fabrication.injection
{
	import flash.utils.Dictionary;
	import flash.utils.getQualifiedClassName;
	
	import org.as3commons.reflect.Field;
	import org.as3commons.reflect.MetaData;
	import org.as3commons.reflect.MetaDataArgument;
	import org.as3commons.reflect.Type;
	import org.puremvc.as3.multicore.interfaces.IFacade;
	import org.puremvc.as3.multicore.utilities.fabrication.Fabrication;
	import org.puremvc.as3.multicore.utilities.fabrication.global.FabricationConfig;
	import org.puremvc.as3.multicore.utilities.fabrication.interfaces.IClassInfoType;
	import org.puremvc.as3.multicore.utilities.fabrication.interfaces.IInjectProcessor;

	public class InjectProcessorFactory
	{
		private static var instance:InjectProcessorFactory ;
		public static function getInstance():InjectProcessorFactory
		{
			if(instance == null){
				instance =  new InjectProcessorFactory();
			}
			return instance;
		}
		
		/**
		 * inject factory
		 */ 
		public function inject(facade:IFacade=null, context:*=null):Array
		{
			if(!FabricationConfig.inject_enabledBool) return [] ;
			
			var qpath:String = getQualifiedClassName(context);
			if(qpath.indexOf(Fabrication.short_packageName) != -1 ) {
				trace("no need to inject : " + qpath)
				return [];	
			}
			
			
			//check have Illegal metadata
			var tp:Type = IClassInfoType(context).getClassInfo();
			var a:Array = tp.fields;
			//no have metadata
			var _haveMetaBool:Boolean = false;
			for each(var field:Field in a) {
				if(field!=null){
					var fieldMetadatas:Array = field.metaData;
					if (fieldMetadatas && fieldMetadatas.length>0) {
						_haveMetaBool = true;
						break;
					}
				}
			}
			
			if(!_haveMetaBool) return [];
			
			var out:Array = [];
			a = FabricationConfig.metaDatas;
			for(var i:int=0;i<a.length;i++){
				out = out.concat(parserMetadata(a[i],i,facade, context))
			}
			
			tp = null;
			
			return out;
		}
		
		
		/**
		 * Runs injection procedure on given context
		 * @return Array of injected field names
		 */
		private function parserMetadata(injectionMetadataTagName:String,ind:Number,facade:IFacade=null, context:*=null):Array
		{
			
			var fieldNames:Array = [];
			var injectionField:InjectionField;
			
			var contextInjectionDataMarkup:String = IClassInfoType(context).getClassInfo().fullName + "_" + injectionMetadataTagName;
			
			var contextInjectionData:Array = [];
			var injectionFields:Array = getInjectionFieldsByInjectionType( IClassInfoType(context).getClassInfo(), injectionMetadataTagName);
			
			for each(injectionField in injectionFields) 
			{
				if(injectionField!=null)
				{
					var fieldName:String = processSingleField(injectionField,ind,facade,context);
					if (fieldName){
						fieldNames[ fieldNames.length ] = fieldName;
					}
				}
			}
			return fieldNames;
		}
		
		/**
		 * Returns info about context properties described by injection metatag
		 * @param injectionType type of injection ( metatag name )
		 * @return Array instance of InjectionField elements
		 * @see org.puremvc.as3.multicore.utilities.fabrication.injection.InjectionField
		 */
		private function getInjectionFieldsByInjectionType(type:Type, injectionMetadataTagName:String):Array
		{
			
			var injectionFields:Array = [];
			var fields:Array = type.fields;
			for each(var field:Field in fields) {
				
				var fieldMetadatas:Array = field.metaData;
				if (fieldMetadatas && fieldMetadatas.length) {
					
					for each(var metadata:MetaData in fieldMetadatas) {
						
						if (metadata.name == injectionMetadataTagName) {
							
							var injectionField:InjectionField = new InjectionField();
							
							injectionField.fieldName = field.name;
							injectionField.fullName  = type.fullName;
							
							// [InjectProxy(name="ReportProxyTwo")]
							var nameArgument:MetaDataArgument = metadata.getArgument( "name" );
							if( nameArgument ){
								injectionField.elementName = nameArgument.value;
							}
							
							var facade_sign:MetaDataArgument = metadata.getArgument("facade");
							if( facade_sign ){
								injectionField.facadeSign = facade_sign.value;
							}
							
							injectionFields.push( injectionField );
								
						}
					}
				}
			}
			return injectionFields;
		}
		
		/**
		 * process single metadata
		 */ 
		private function processSingleField(injectionField:InjectionField, ind:Number,facade:IFacade=null, context:*=null):String
		{
			if (IClassInfoType(context).haveInjectName( injectionField.fieldName )) return null;
			
			// retrieve element name
			// elementName :: AppPropActionMediator
			//injectionField.fieldName ::: propProxy
			var elementName:String = injectionField.elementName
			
			//not yet register
			//put to cache
			var temp_facade:IFacade = IInjectProcessor(FabricationConfig.processors[ind]).elementExist(injectionField,facade);
			if (elementName && temp_facade == null)
			{
				NoPatternCache.add(injectionField,facade,ind,context)
				onNoPatternElementAvaiable(elementName,injectionField.fullName);
				return null;
			}
			
			//every processor handle
			var patternElement:Object = IInjectProcessor(FabricationConfig.processors[ind]).processor(injectionField , temp_facade);
			if (patternElement)
			{
				IClassInfoType(context).setInjectName( injectionField.fieldName , patternElement);
			}
			else
			{
				NoPatternCache.add(injectionField,facade,ind,context)
			}
			return injectionField.fieldName;
		}
				
		/**
		 * Function handler invoked when there is no element present with given name
		 * @param elementName name of element
		 */
		protected function onNoPatternElementAvaiable(elementName:String,fullName:String):void
		{
			//trace("Cannot resolve injection element [ " + elementName + " ] at [ " + fullName + " ]");
		}
		
	}
}